<?php

declare(strict_types=1);

namespace Drupal\cas_user_ban\Hook;

use Drupal\cas\Service\CasUserManager;
use Drupal\cas_user_ban\CasUserBanManagerInterface;
use Drupal\cas_user_ban\Event\FilterUserCancelMethodEvent;
use Drupal\cas_user_ban\Exception\ExistingBanException;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Psr\EventDispatcher\EventDispatcherInterface;

/**
 * Class to for user cancel.
 */
final class UserCancelFormHooks {

  use StringTranslationTrait;
  use DependencySerializationTrait;

  public function __construct(
    protected EventDispatcherInterface $eventDispatcher,
    protected CasUserBanManagerInterface $casUserBanManager,
    protected CasUserManager $casUserManager,
    protected AccountProxyInterface $currentUser,
    protected MessengerInterface $messenger,
  ) {}

  /**
   * Implements hook_form_FORM_ID_alter().
   *
   * @param array $form
   *   The build form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  #[Hook('form_user_cancel_form_alter')]
  public function alter(array &$form, FormStateInterface $form_state): void {
    $uid = (int) $form['uid']['#value'];

    // Users should not be able to ban themselves.
    if ($this->currentUser->getAccount()->id() == $uid) {
      return;
    }

    // Only users with CAS Username should be allowed to be banned.
    $cas_username = $this->casUserManager->getCasUsernameForAccount($uid);
    if ($cas_username === FALSE) {
      return;
    }

    // To make the ban work independently of the submit execution, like user
    // deletion before the ban, we store the cas_username.
    $form_state->set('cas_username', $cas_username);

    // Allow to alter the allowed cancel methods.
    $event = new FilterUserCancelMethodEvent([
      'user_cancel_reassign',
      'user_cancel_delete',
    ]);
    $this->eventDispatcher->dispatch($event, FilterUserCancelMethodEvent::NAME);

    $filter_methods = $event->getAllowedMethods();
    $allowed_methods = Element::getVisibleChildren($form['user_cancel_method']);
    $allowed_methods = array_intersect($filter_methods, $allowed_methods);

    if (empty($allowed_methods)) {
      return;
    }
    $form_state->set('cas_ban_allowed_methods', $allowed_methods);

    $state_allowed_methods = [];
    foreach ($allowed_methods as $index => $cancel_method) {
      $state_allowed_methods[] = ['value' => $cancel_method];
      if ($index !== array_key_last($allowed_methods)) {
        $state_allowed_methods[] = 'or';
      }
    }

    // Add form element to mark the user to be banned.
    $cancel_method = $form_state->getValue('user_cancel_method', $form['user_cancel_method']['#default_value']);
    $form['ban_cas_username'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Prevent user from re-creating the account.'),
      '#description' => $this->t("The username will be added to the banned list of CAS users and they won't be able to re-create the account."),
      '#default_value' => in_array($cancel_method, $allowed_methods),
      '#states' => [
        'visible' => [
          ':input[name="user_cancel_method"]' => $state_allowed_methods,
        ],
        'checked' => [
          ':input[name="user_cancel_method"]' => $state_allowed_methods,
        ],
      ],
    ];

    $form['actions']['submit']['#submit'][] = [$this, 'banSubmit'];
  }

  /**
   * Ban the user based in the CAS Username.
   *
   * @param array $form
   *   The build form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function banSubmit(array $form, FormStateInterface $form_state): void {
    $to_be_banned = $form_state->getValue('ban_cas_username');
    $cancel_method = $form_state->getValue('user_cancel_method');
    $cas_username = $form_state->get('cas_username');
    $allowed_methods = $form_state->get('cas_ban_allowed_methods', []);

    // Conditions to match in order to ban the user:
    // - User is marked to be banned.
    // - Cancel method is part of allowed methods.
    if ($to_be_banned && in_array($cancel_method, $allowed_methods)) {
      try {
        $this->casUserBanManager->add($cas_username);
        $this->messenger->addStatus($this->t('The user %username has been banned.', ['%username' => $cas_username]));
      }
      catch (ExistingBanException $e) {
        $this->messenger->addWarning($this->t('The user %username is already banned.', ['%username' => $cas_username]));
      }
    }
  }

}
